@@ -1,5 +1,6 @@
 package Module::CPANfile::Environment;
 use strict;
+use warnings;
 use Module::CPANfile::Result;
 use Carp ();
 
@@ -12,36 +13,41 @@ my @bindings = qw(
 
 my $file_id = 1;
 
-sub import {
-    my($class, $result_ref) = @_;
+sub new {
+    my($class, $file) = @_;
+    bless {
+        file => $file,
+    }, $class;
+}
+
+sub bind {
+    my $class = shift;
     my $pkg = caller;
 
-    $$result_ref = Module::CPANfile::Result->new;
+    my $result = Module::CPANfile::Result->new;
     for my $binding (@bindings) {
         no strict 'refs';
-        *{"$pkg\::$binding"} = sub { $$result_ref->$binding(@_) };
+        *{"$pkg\::$binding"} = sub { $result->$binding(@_) };
     }
+
+    return $result;
 }
 
 sub parse {
-    my $file = shift;
-
-    my $code = do {
-        open my $fh, "<", $file or die "$file: $!";
-        join '', <$fh>;
-    };
+    my($self, $code) = @_;
 
     my($res, $err);
 
     {
         local $@;
-        $res = eval sprintf <<EVAL, $file_id++;
-package Module::CPANfile::Sandbox%d;
+        $file_id++;
+        $res = eval <<EVAL;
+package Module::CPANfile::Sandbox$file_id;
 no warnings;
 my \$_result;
-BEGIN { import Module::CPANfile::Environment \\\$_result };
+BEGIN { \$_result = Module::CPANfile::Environment->bind }
 
-# line 1 "$file"
+# line 1 "$self->{file}"
 $code;
 
 \$_result;
@@ -49,7 +55,7 @@ EVAL
         $err = $@;
     }
 
-    if ($err) { die "Parsing $file failed: $err" };
+    if ($err) { die "Parsing $self->{file} failed: $err" };
 
     return $res;
 }
